@@ -1,8 +1,8 @@
-/* A Bison parser, made by GNU Bison 3.8.2.  */
+/* A Bison parser, made by GNU Bison 3.5.  */
 
 /* Bison interface for Yacc-like parsers in C
 
-   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2021 Free Software Foundation,
+   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2019 Free Software Foundation,
    Inc.
 
    This program is free software: you can redistribute it and/or modify
@@ -16,7 +16,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* As a special exception, you may create a larger work that contains
    part or all of the Bison parser skeleton and distribute that work
@@ -31,9 +31,8 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
-/* DO NOT RELY ON FEATURES THAT ARE NOT DOCUMENTED in the manual,
-   especially those whose name start with YY_ or yy_.  They are
-   private implementation details that can be changed or removed.  */
+/* Undocumented macros, especially those whose name start with YY_,
+   are private implementation details.  Do not rely on them.  */
 
 #ifndef YY_AUGL_PARSER_H_INCLUDED
 # define YY_AUGL_PARSER_H_INCLUDED
@@ -45,41 +44,32 @@
 extern int augl_debug;
 #endif
 
-/* Token kinds.  */
+/* Token type.  */
 #ifndef YYTOKENTYPE
 # define YYTOKENTYPE
   enum yytokentype
   {
-    YYEMPTY = -2,
-    YYEOF = 0,                     /* "end of file"  */
-    YYerror = 256,                 /* error  */
-    YYUNDEF = 257,                 /* "invalid token"  */
-    DQUOTED = 258,                 /* DQUOTED  */
-    REGEXP = 259,                  /* REGEXP  */
-    LIDENT = 260,                  /* LIDENT  */
-    UIDENT = 261,                  /* UIDENT  */
-    QIDENT = 262,                  /* QIDENT  */
-    ARROW = 263,                   /* ARROW  */
-    KW_MODULE = 264,               /* KW_MODULE  */
-    KW_AUTOLOAD = 265,             /* KW_AUTOLOAD  */
-    KW_LET = 266,                  /* KW_LET  */
-    KW_LET_REC = 267,              /* KW_LET_REC  */
-    KW_IN = 268,                   /* KW_IN  */
-    KW_STRING = 269,               /* KW_STRING  */
-    KW_REGEXP = 270,               /* KW_REGEXP  */
-    KW_LENS = 271,                 /* KW_LENS  */
-    KW_TEST = 272,                 /* KW_TEST  */
-    KW_GET = 273,                  /* KW_GET  */
-    KW_PUT = 274,                  /* KW_PUT  */
-    KW_AFTER = 275                 /* KW_AFTER  */
+    DQUOTED = 258,
+    REGEXP = 259,
+    LIDENT = 260,
+    UIDENT = 261,
+    QIDENT = 262,
+    ARROW = 263,
+    KW_MODULE = 264,
+    KW_AUTOLOAD = 265,
+    KW_LET = 266,
+    KW_LET_REC = 267,
+    KW_IN = 268,
+    KW_STRING = 269,
+    KW_REGEXP = 270,
+    KW_LENS = 271,
+    KW_TEST = 272,
+    KW_GET = 273,
+    KW_PUT = 274,
+    KW_AFTER = 275
   };
-  typedef enum yytokentype yytoken_kind_t;
 #endif
-/* Token kinds.  */
-#define YYEMPTY -2
-#define YYEOF 0
-#define YYerror 256
-#define YYUNDEF 257
+/* Tokens.  */
 #define DQUOTED 258
 #define REGEXP 259
 #define LIDENT 260
@@ -117,7 +107,7 @@ union YYSTYPE
   int            intval;
   enum quant_tag quant;
 
-#line 121 "parser.h"
+#line 111 "parser.h"
 
 };
 typedef union YYSTYPE YYSTYPE;
@@ -141,9 +131,7 @@ struct YYLTYPE
 
 
 
-
 int augl_parse (struct term **term, yyscan_t scanner);
-
 /* "%code provides" blocks.  */
 #line 46 "parser.y"
 
@@ -156,6 +144,6 @@ struct state {
 };
 
 
-#line 160 "parser.h"
+#line 148 "parser.h"
 
 #endif /* !YY_AUGL_PARSER_H_INCLUDED  */
